/// The AssetService is a class / static library that manages on-file game assets.
/// This class service provides asynchronous loading of game content, with introspection for hooking to a loading menu.

#pragma once

#include <string>
#include <unordered_map>
#include <filesystem>
#include <iostream>
#include <JGL/types/Texture.h>
#include <JGL/types/Font.h>

namespace CaveGame::Client
{

    enum AssetType { DATA, TEXT, AUDIO, MODEL, TEXTURE, FONT, SHADER };
    enum AssetLifestyle { STATIC, STREAMED};

    struct Asset {
        std::string name;
        std::string metadata;
        std::string type;

    };

    class AssetService {
    private:
    public:

        static AssetService* Get();


        JGL::Texture* player_sprite;
        JGL::Texture* explosion_sprite;

        AssetService();

        ~AssetService() {}

        void TempLoadSimple();

        /// Performs one "Load Cycle" on the current thread. This means we will attempt to load a number of queued items until we've spent at least maxTTL seconds.
        void LoadCycle(float maxTTL = 1e-3f);

        unsigned int TotalAssetsQueued() const;
        unsigned int TotalAssetsLoaded() const;
        unsigned int AssetsToBeLoaded() const;

        JGL::Texture* GetTexture(const std::string& textureName) {
            return textures[textureName];
        }
        JGL::Font* GetFont(const std::string& fontName) {
            return fonts[fontName];
        }

        void EnqueueTexture(const std::string& texName);
        void EnqueueFont(const std::string& fontName);

        void EnqueueDefaultAssetsFolder();

        void EnqueueContents(const std::filesystem::path& folder) {
            for (const auto& entry : std::filesystem::recursive_directory_iterator(folder)) {
                if (entry.is_regular_file()) {
                    std::cout << entry << std::endl;
                }
            }
        }

    protected:
        std::unordered_map<std::string, JGL::Font*> fonts;
        std::unordered_map<std::string, JGL::Texture*> textures;

    };
}
