#pragma once

#include <Core/World.hpp>
#include <Core/Chunk.hpp>
#include <JGL/JGL.h>
#include <Client/Camera2D.hpp>
#include <Core/SimpleAABBSolver.hpp>
#include <thread>
#include <Core/ConcurrentQueue.hpp>
#include <Core/ThreadPool.hpp>

namespace CaveGame::Client
{
    using Core::ConcurrentQueue;
    using namespace std::chrono_literals;

    class AsyncChunkRenderer : public ThreadPool
    {
    public:

    };

    /// An extension of the world object that provides rendering for the game world.
    class LocalWorld : public CaveGame::Core::World
    {
    public:
        Camera2D camera;
        Vector2 mouse_pos;
    public:
        JGL::Font font;
        LocalWorld(const std::string& world_name, int seed, bool overwrite);
        void PostInit();
        void Draw();
        void Update(float elapsed) override;
        void RefreshAll() override;

        unsigned int GetRenderTargetCount() const;


    protected:

        float check_chunks_timer = 0.f;

        std::unordered_map<Vector2, JGL::RenderTarget*> cached_chunk_sprites;
        JGL::RenderTarget* canvas_render_target;


        /// Determines whether a given chunk coordinate lies within the viewable space, with an optional 'oversize' parameter.
        /// @param extraChunkRadius Specifies how many extra chunks can be considered within the viewport when they are in fact, just outside of the viewport.
        /// @note This is used to overdraw
        bool IsChunkCellWithinViewport(const Vector2& coords, int extraChunkRadius = 0) const;

        void RenderChunk(const Vector2& coords);

        void LookForChunksNeedLoading();
        void LookForChunksNeedUnloading();

        void RenderChunkTexture(const Vector2 &coords, JGL::RenderTarget* destination, Core::Chunk* chunk);

        /// Render chunk boundaries and relative coordinates, around the camera.
        void DrawChunkGrid() const;
        /// Checks for missing, or out-of-date, cached sprites of chunks, and renders them.
        void CheckCachedChunkSprites();

        static Color4 GetSkyColorInterpolatedForTimeOfDay(float time);

        static Color4 GetSkyColorBaseForTimeOfDay(float time);

        void DrawSky();
    };
}