#pragma once

#include <J3ML/LinearAlgebra/Vector2.hpp>
#include "SceneManager.hpp"
#include <rewindow/types/key.h>

namespace CaveGame::Client
{

    class SceneManager;

    /// The scene class is a self-contained environment for logic, input, and rendering.
    class Scene
    {
    public:
        Scene();
        virtual ~Scene() = default;

        // TODO if the overriding function *has* to call the original then you're not doing it right.
        // Whether or not a scene is active should return if CurrentScene == this - Redacted.

        /// This function is called by the SceneManager when a scene is changed.
        /// @note Make sure to call this base when overriding in derived Scene classes.
        virtual void Load();
        /// This function is called by the SceneManager when a scene is changed.
        /// @note Make sure to call this base when overriding in derived Scene classes.
        virtual void Unload();

        /// This function is called by the SceneManager every frame. It is not called when the scene is inactive.
        virtual void Update(float elapsed) = 0;

        /// This function is called by the SceneManager every frame. It is not called when the scene is inactive.
        virtual void Draw() = 0;

        /// This function is called by the SceneManager when the game window changes dimensions.
        virtual void PassWindowSize(const Vector2& size);

        /// This function is called by the SceneManager when user mouse button input is detected.
        /// @param mouse_btn A number indicating the mouse button ID.
        /// @param pressed True if the button was pressed, false if the button was released.
        virtual void PassMouseInput(unsigned int mouse_btn, bool pressed) {}

        /// This function is called by the SceneManager when user key input is detected.
        /// @param key An enumeration indicating the key.
        /// @param pressed True if the key was pressed, false if the key was released.
        virtual void PassKeyInput(Key key, bool pressed) {}

        /// This function is called by the SceneManagert when user mouse movement is detected.
        /// @param pos The up-to-date mouse position.
        virtual void PassMouseMovement(const Vector2& pos) {}

        /// Returns whether this scene is currently the "Active" scene.
        [[nodiscard]] bool Active() const;
    protected:
        bool active;
        Vector2 window_size;
    private:
    };
}

