#include <Client/Camera2D.hpp>
#include <rewindow/inputservice.hpp>

namespace CaveGame::Client
{
    Camera2D::Camera2D() {

    }

    void Camera2D::UpdateFreecamControls(float elapsed)
    {
        // TODO: implement freecam panning via mouse.

        if (InputService::IsKeyDown(Keys::LeftArrow))
            MoveLeft();
        if (InputService::IsKeyDown(Keys::RightArrow))
            MoveRight();
        if (InputService::IsKeyDown(Keys::UpArrow))
            MoveUp();
        if (InputService::IsKeyDown(Keys::DownArrow))
            MoveDown();

        if (InputService::IsKeyDown(Keys::LeftBracket))
            Rotate(-5.f*elapsed);
        if (InputService::IsKeyDown(Keys::RightBracket))
            Rotate(5.f*elapsed);

        if (InputService::IsKeyDown(Keys::Minus))
            ZoomOut(elapsed);
        if (InputService::IsKeyDown(Keys::Equals))
            ZoomIn(elapsed);
    }

    void Camera2D::Update(float elapsed) {

        if (freecam)
            UpdateFreecamControls(elapsed);

        if (lerp_to_target)
            position = Vector2::Lerp(position, target, Math::Min(elapsed*lerp_factor, 1.f));
        else
            position = target;

        // If **really** close, snap to the final position to stabilize.
        if (position.DistanceSq(target) < 1e-3f)
        {
            position = target;
        }


    }

    AABB2D Camera2D::Viewport() const { return AABB2D(position - (viewport_size/2.f), position+(viewport_size/2.f)); }

    AABB2D Camera2D::ScaledViewport() const {
        return AABB2D(
                position - ScaledHalfSizeOffset(),
                position + ScaledHalfSizeOffset());
    }

    float Camera2D::Zoom() const { return zoom;}

    void Camera2D::Zoom(float val) {
        zoom = val;
        zoom = Math::Clamp(zoom, 0.125f, 50.f);
    }

    void Camera2D::ZoomIn(float amt) {
        // Scale zoom amount to be relative to current scaling.
        amt *= zoom;

        Zoom(Zoom()+amt);

    }

    void Camera2D::ZoomOut(float amt)
    {
        // Scale zoom amount to be relative to current scaling.
        amt *= zoom;

        Zoom(Zoom()-amt);

    }

    float Camera2D::Rotation() const { return rotation; }

    void Camera2D::Rotation(float val) { rotation = val; }

    void Camera2D::Rotate(float angles) { rotation+=angles;}

    Vector2 Camera2D::ScreenToWorld(const Vector2 &device_coordinates) const {
        Vector2 pos = ScaledTopLeft() + (device_coordinates/zoom);
        return pos;
    }

    Vector2 Camera2D::Position() const { return position; }

    void Camera2D::MoveLeft(float rate) {
        target -= Vector2(speed*rate, 0);
    }

    void Camera2D::MoveRight(float rate) {
        target += Vector2(speed*rate, 0);
    }

    void Camera2D::MoveUp(float rate) {
        target -= Vector2(0, speed*rate);
    }

    void Camera2D::MoveDown(float rate) {
        target += Vector2(0, speed*rate);
    }

    void Camera2D::PassViewportSize(const Vector2 &size) {
        viewport_size = size;
    }

    Vector2 Camera2D::TopLeft() const {
        return position - HalfSizeOffset();
    }

    Vector2 Camera2D::ScaledTopLeft() const {
        return position - (ScaledHalfSizeOffset());
    }

    Vector2 Camera2D::Center() const {
        return position;
    }

    Vector2 Camera2D::HalfSizeOffset() const {
        return viewport_size/2.f;
    }

    Vector2 Camera2D::ScaledHalfSizeOffset() const {
        return (viewport_size / zoom) / 2.f;
    }


    Vector2 Camera2D::Target() const { return target;}

    void Camera2D::Target(const Vector2 &pos) { target = pos; }

    float Camera2D::PositionSmoothingFactor() const { return lerp_factor;}

    void Camera2D::PositionSmoothingFactor(float smoothing) { lerp_factor = smoothing;}

    bool Camera2D::PositionSmoothingEnabled() const { return lerp_to_target;}

    void Camera2D::PositionSmoothingEnabled(bool enabled) { lerp_to_target = enabled;}

    void Camera2D::Position(const Vector2 &newPos) {
        position = newPos;
    }

    float Camera2D::MoveSpeed() const { return speed;}

    void Camera2D::MoveSpeed(float value) {
        speed = value;
    }

    AABB2D Camera2D::ScaledViewportOversized(float oversize) const {
        return AABB2D(
                position - ScaledHalfSizeOffset() - Vector2(oversize),
                position + ScaledHalfSizeOffset() + Vector2(oversize*2));
    }
}