#include <Client/Splash.hpp>
#include <JGL/JGL.h>

CaveGame::Client::Splash::Splash() : Scene()
{

    //font = JGL::Font();

    ComputeMatrixGlyphTable();

}


void CaveGame::Client::Splash::ComputeMatrixGlyphTable()
{
    J3ML::Algorithm::RNG rng;

    for (int i = 0; i < 128; i++)
        scroll_offsets.push_back(rng.Float(0, 300.5));

    for (int i = 0; i < column_textures.size(); i++)
    {
        matrix.push_back(std::vector<char>());
        for (int j = 0; j < 100; j++)
        {
            if (rng.FloatNeg1_1() > 0)
                matrix[i].push_back('1');
            else
                matrix[i].push_back('0');
        }
    }
}


void CaveGame::Client::Splash::ComputeMatrixTextureCache()
{
    auto glyph_measurement = JGL::Fonts::Jupiteroid.MeasureString("0", 16);
    column_width = glyph_measurement.x;

    for (int i = 0; i < column_textures.size(); i++)
    {

        Vector2i column_size = {(int) glyph_measurement.x, (int) glyph_measurement.y * 100};
        auto* column = new JGL::RenderTarget(column_size, {0, 0, 0, 0}, false);

        JGL::J2D::Begin(column, true);

        for (int col = 0; col < 50; col++) {
            Color4 text_col = Color4(32, 192, 92, 255 - (col*4));
            JGL::J2D::DrawString(text_col, std::to_string(rand() % 2), 0, glyph_measurement.y * col, 1, 16);
        }
        JGL::J2D::End();

        column_textures[i] = column;
    }
}


void CaveGame::Client::Splash::DrawProgressBar()
{
    auto screen_dimensions = window_size;

    float bar_length = screen_dimensions.x-20;
    float bar_height = 20;
    float bar_pos_x = 10;
    float bar_pos_y = (screen_dimensions.y-bar_height)-10;

    Vector2 bar_pos = {bar_pos_x, bar_pos_y};

    JGL::J2D::FillRect(Colors::Gray, bar_pos, {bar_length, bar_height});

    float progress_bar_length = bar_length * load_percent;

    JGL::J2D::FillRect(Colors::White, bar_pos, {progress_bar_length, bar_height});
}


void CaveGame::Client::Splash::DrawMatrix()
{

    for (int i = 0; i < window_size.x/column_width; i++)
    {
        float scroll = (scroll_offsets[i] * load_percent);
        auto column = column_textures[i%column_width];
        JGL::J2D::DrawSprite(*column, {static_cast<float>(i*16),scroll});
    }

}

void CaveGame::Client::Splash::Draw()
{

    auto texture_dimensions = splash->GetDimensions();
    auto screen_dimensions = window_size;

    Vector2 aspect = {
        screen_dimensions.x / texture_dimensions.x,
        screen_dimensions.y / texture_dimensions.y
    };

    Vector2 middle = screen_dimensions/2.f;

    // TODO: Implement draw-point offset to maintain sensible aspect ratio on the image.


    JGL::J2D::Begin();
    DrawMatrix();
    //
    JGL::J2D::DrawSprite(*splash, {0, 0}, 0.f, {0, 0}, aspect);
    DrawProgressBar();


    JGL::J2D::End();
}

void CaveGame::Client::Splash::Update(float elapsed)
{

    if (load_percent < 1)
        load_percent += elapsed/2.f;
    else
        load_percent = 1;

    splash_timer -= elapsed;

}

void CaveGame::Client::Splash::Load() {
    column_textures.fill(nullptr);
    splash = new JGL::Texture("assets/textures/redacted.png");
    ComputeMatrixTextureCache();

    Scene::Load();

}

void CaveGame::Client::Splash::Unload() {
    for (auto& r : column_textures)
        delete r;
    delete splash;

    column_textures.fill(nullptr);
    Scene::Unload();
}

bool CaveGame::Client::Splash::SplashComplete() const {
    return splash_timer < 0;
}

CaveGame::Client::Splash::~Splash() {
    Splash::Unload();
}
