/// CaveGame - A procedural 2D platformer sandbox.
/// Created by Josh O'Leary @ Redacted Software, 2020-2024
/// Contact: josh@redacted.cc
/// Contributors: william@redacted.cc maxi@redacted.cc
/// This work is dedicated to the public domain.

/// @file main.cpp
/// @desc The Client App's main file. Creates and runs the Client App window.
/// @edit 11/1/2024
/// @auth Josh O'Leary

#ifdef STEAM_BUILD
#include <steam_api.h>
#endif

#include <JGL/JGL.h>
#include <ClientApp/CaveGameWindow.hpp>
#include <Core/Loggers.hpp>
#include <rewindow/logger/logger.h>
#include <Core/Tile.hpp>
#include <JGL/logger/logger.h>

int main(int argc, char** argv) {

    JGL::Logger::Warning.EnableConsole(false);
    JGL::Logger::Debug.EnableConsole(false);
    ReWindow::Logger::Debug.EnableConsole(false);
    ReWindow::Logger::Debug.EnableFile(false);
    CaveGame::Logs::Info.IncludeLocation(false);
    CaveGame::Logs::Info("Starting client program.");

    // TODO: Steam
#ifdef STEAM_BUILD
    CaveGame::Logs::Info.Log("Running steam build.");
    bool steam_success = SteamAPI_Init();
#endif


    auto* window = new CaveGame::ClientApp::CaveGameWindow("Re-CaveGame", 800, 600);
    //window->SetResizable(true);
    window->Run();

    delete window;

#ifdef STEAM_BUILD
    SteamAPI_Shutdown();
#endif

    CaveGame::Logs::Info.Log("Exiting client program.");

    return 0;
}
