#pragma once
#include "Entity.hpp"


namespace CaveGame::Core
{
    class PhysicsEntity : public Entity {
    public:
        explicit PhysicsEntity(const Vector2& spawnPoint);

        [[nodiscard]] Vector2 Velocity() const;

        [[nodiscard]] Vector2 EstimatedNextPosition() const;

        void Accelerate(const Vector2& vector);

        virtual void PhysicsUpdate(float elapsed);

        void CollisionTest(ITileMap* map, float elapsed) override
        {

            for (int x = -1; x <= bounding_box.x; x++) {
                for (int y = -1; y <= bounding_box.y; y++) {
                    Vector2 tileBoxPos = Vector2(Math::Floor(position.x), Math::Floor(position.y)) + Vector2(x,y);

                    TileID id = map->GetTile(tileBoxPos.x, tileBoxPos.y);

                    if (id == TileID::VOID || id == TileID::AIR)
                        continue;

                    Vector2 entity_centroid = position + (bounding_box / 2.f);
                    Vector2 entity_halfbox = bounding_box / 2.f;

                    Vector2 tile_centroid = tileBoxPos + Vector2{0.5f, 0.5f};
                    Vector2 tile_halfbox = Vector2{0.5f, 0.5f};


                    if (Solver::AABB2Dvs(entity_centroid, entity_halfbox, tile_centroid, tile_halfbox)) {
                        Vector2 separation = Solver::SolveAABB(entity_centroid, entity_halfbox, tile_centroid, tile_halfbox);

                        Vector2 normal = Solver::GetNormalForAABB(separation, velocity);

                        next_position += separation;

                        // Touched top.
                        if (normal.y == -1)
                            velocity.y = 0;

                        if (normal.y == 1)
                            velocity.y = -velocity.y;

                       //if (normal.x != 0)
                       //     velocity = {-velocity.x, velocity.y};

                        // TODO: Refine condition for "step-up"s
                        if (normal.x != 0 && velocity.y == 0)
                            next_position.y -= 1;

                        next_position += separation;
                        return;
                    }
                }
            }
        }

        void Update(float elapsed) override {

            // TODO: Sophisticated mechanism to maintain locked-timestep, multiple-iteration physics steps.
            PhysicsUpdate(elapsed);

        }

    protected:
        Vector2 velocity;
        Vector2 next_position;
        float mass;
    private:
    };

}
