#include <Core/World.hpp>
#include <filesystem>
#include <iostream>
#include <fstream>
#include <format>
#include <chrono>
#include <Core/Loggers.hpp>

#include <JJX/JJX.hpp>

namespace CaveGame::Core
{
    void CaveGame::Core::World::Update(float elapsed) {
        time_of_day += elapsed;

        time_of_day = std::fmod(time_of_day, 24*60);

        autosave_timer += elapsed;

        if (autosave_timer >= PeriodicAutosaveIntervalSeconds)
        {
            AutoSave();
            autosave_timer = 0;
        }

        if (simulate_tiles) {
            tile_ticc_counter+=elapsed;

            if (tile_ticc_counter >  1.f / TileTiccRate) {
                tile_ticc_counter -= 1.f / TileTiccRate;
                for (const auto& [coords, chunk] : loaded_chunks) {
                    DoRandomTileTick(coords, chunk);
                    DoForcedTileTick(coords, chunk);

                }

                //DoRandomTileTicks();
                //DoForcedTileTicks();
            }
        }

        for (auto* entity : entities) {
            entity->Update(elapsed);
            entity->CollisionTest(this, elapsed);

        }
    }

    TileID World::GetTile(int x, int y) const {
        /*
        float chunkX = Math::Floor((float)x / Chunk::ChunkSize);
        float chunkY = Math::Floor((float)y / Chunk::ChunkSize);
        float tileX = Math::Mod(x, Chunk::ChunkSize);
        float tileY = Math::Mod(y, Chunk::ChunkSize);

        if (tileX < 0)
            tileX = Chunk::ChunkSize + tileX;
        if (tileY < 0)
            tileY = Chunk::ChunkSize + tileY;

        Vector2 coords = Vector2(chunkX, chunkY);

        if (HasChunkAtCell(coords))
            return loaded_chunks.at(coords)->GetTile(tileX, tileY);

        return TileID::VOID;*/

        Vector2 chunkCoords = GetChunkCoordinatesAtCell(x, y);
        Vector2 tileCoords = GetTileCoordinatesAtCell(x, y);

        if (HasChunkAtCell(chunkCoords))
            return loaded_chunks.at(chunkCoords)->GetTile(tileCoords.x, tileCoords.y);

        return TileID::VOID;
    }

    void World::SetTile(int x, int y, TileID t, bool flag_update) {
        /*
        float chunkX = Math::Floor((float)x / Chunk::ChunkSize);
        float chunkY = Math::Floor((float)y / Chunk::ChunkSize);
        float tileX = Math::Mod(x, Chunk::ChunkSize);
        float tileY = Math::Mod(y, Chunk::ChunkSize);

        if (tileX < 0)
            tileX = Chunk::ChunkSize + tileX;
        if (tileY < 0)
            tileY = Chunk::ChunkSize + tileY;

        const Vector2 coords = Vector2(chunkX, chunkY);

        if (HasChunkAtCell(coords))
        {
            loaded_chunks.at(coords)->SetTile(tileX, tileY, t, flag_update);

            //SetTileUpdateFlag(x, y, true);
            if (flag_update) {
                SetTileUpdateFlag(x, y, true);
                SetTileUpdateFlag(x, y-1, true);
                SetTileUpdateFlag(x, y+1, true);
                SetTileUpdateFlag(x-1, y, true);
                SetTileUpdateFlag(x+1, y, true);
            }
        }*/


        Vector2 chunkCoords = GetChunkCoordinatesAtCell(x, y);
        Vector2 tileCoords = GetTileCoordinatesAtCell(x, y);

        if (HasChunkAtCell(chunkCoords)) {
            loaded_chunks.at(chunkCoords)->SetTile(tileCoords.x, tileCoords.y, t, flag_update);
            if (flag_update) {
                SetTileUpdateFlag(x, y, true);
                SetTileUpdateFlag(x, y-1, true);
                SetTileUpdateFlag(x, y+1, true);
                SetTileUpdateFlag(x-1, y, true);
                SetTileUpdateFlag(x+1, y, true);
            }
        }

    }

    void World::SwapTile(int source_x, int source_y, int dest_x, int dest_y, bool flag_update) {
        auto source = GetTile(source_x, source_y);
        auto dest = GetTile(dest_x, dest_y);
        SetTile(source_x, source_y, dest, flag_update);
        SetTile(dest_x, dest_y, source, flag_update);
    }

    bool World::GetTileSimulationEnabled() const { return simulate_tiles; }

    void World::SetTileSimulationEnabled(bool enabled) {
        simulate_tiles = enabled;
    }

    Chunk* World::GetChunkAtCell(const Vector2& cell) {
        if (ValidCoords(cell))
            return loaded_chunks.at(cell);
        return nullptr;
    }

    bool World::ValidCoords(const Vector2& cell) const {
        if (Math::Floor(cell.x) != cell.x || Math::Floor(cell.y) != cell.y) {
            std::cerr << "Invalid cell coordinate: " << cell << ", must be whole numbers!";
            return false;
        }

        return true;
    }

    bool World::HasChunkAtCell(const Vector2 &cell) const {
        if (ValidCoords(cell))
            return loaded_chunks.contains(cell);
        return false;
    }

    std::unordered_map<Vector2, Chunk*> World::GetChunkList() { return loaded_chunks; }

    void World::SetTileUpdateFlag(int x, int y, bool flag) {
        /*
        float chunkX = Math::Floor((float)x / Chunk::ChunkSize);
        float chunkY = Math::Floor((float)y / Chunk::ChunkSize);

        float tileX = Math::Mod(x, Chunk::ChunkSize);
        float tileY = Math::Mod(y, Chunk::ChunkSize);

        if (tileX < 0)
            tileX = Chunk::ChunkSize + tileX;
        if (tileY < 0)
            tileY = Chunk::ChunkSize + tileY;


        const Vector2 coords = Vector2(chunkX, chunkY);

        if (HasChunkAtCell(coords))
            loaded_chunks.at(coords)->SetTileUpdateFlag(tileX, tileY, flag);*/


        const Vector2 chunkCoords = GetChunkCoordinatesAtCell(x, y);
        Vector2 tileCoords = GetTileCoordinatesAtCell(x, y);

        if (HasChunkAtCell(chunkCoords))
            loaded_chunks.at(chunkCoords)->SetTileUpdateFlag(tileCoords.x, tileCoords.y, flag);

    }

    bool World::GetTileUpdateFlag(int x, int y) const {
        /*
        float chunkX = Math::Floor((float)x / Chunk::ChunkSize);
        float chunkY = Math::Floor((float)y / Chunk::ChunkSize);

        float tileX = Math::Mod(x, Chunk::ChunkSize);
        float tileY = Math::Mod(y, Chunk::ChunkSize);

        if (tileX < 0)
            tileX = Chunk::ChunkSize + tileX;
        if (tileY < 0)
            tileY = Chunk::ChunkSize + tileY;

        Vector2 coords = Vector2(chunkX, chunkY);


        if (HasChunkAtCell(coords))
            return loaded_chunks.at(coords)->GetTileUpdateFlag(tileX, tileY);

        return false;*/

        Vector2 chunkCoords = GetChunkCoordinatesAtCell(x, y);
        Vector2 tileCoords = GetTileCoordinatesAtCell(x, y);

        if (HasChunkAtCell(chunkCoords))
            return loaded_chunks.at(chunkCoords)->GetTileUpdateFlag(tileCoords.x, tileCoords.y);

    }

    Vector2 World::ToUnitDirection(float rotation) {
        return {std::cos(rotation), std::cos(rotation)};
    }


    std::string read_text_file(const std::filesystem::path& path)
    {
        std::ifstream file(path, std::ios::binary);
        if (!file)
            throw std::runtime_error("We couldn't find the file: " + path.string());

        std::streamsize file_size;
        file.seekg(0, std::ios::end);
        file_size = file.tellg();
        file.seekg(0, std::ios::beg);

        std::string file_content(file_size, '\0');
        file.read(&file_content[0], file_size);
        file.close();

        return file_content;

    }

    void write_text_file(const std::filesystem::path& path, const std::string& text)
    {
        std::ofstream file(path, std::ios::binary);
        if (!file)
            throw std::runtime_error("We couldn't find the file: " + path.string());

        file << text;
        file.close();
    }


    std::string timestamp()
    {
        return std::format("{:%F %T}", std::chrono::system_clock::now());
    }

    World::World(const std::string &worldName, int seed, bool overwrite) : world_name(worldName),
                                                                           generator(seed) {
        namespace fs = std::filesystem;
        namespace json = JJX::json;

        /// Create "worlds" directory if not already there.
        if (!fs::exists(worlds))
            fs::create_directory(worlds);

        fs::path current_world_path = worlds / worldName;

        if (overwrite) {
            if (fs::exists(current_world_path))
                fs::remove_all(current_world_path);
        }

        /// Create directory for this world if not already there.
        if (!fs::exists(current_world_path))
            fs::create_directory(current_world_path);

        /// Create directory for chunks if not already there.
        if (!fs::exists(current_world_path / "chunks"))
            fs::create_directory(current_world_path / "chunks");

        fs::path metadata_path = current_world_path / "meta.json";

        if (!fs::exists(metadata_path)) {
            /// Fill metadata file with base if it is not present.

            auto world_meta_template = json::object({
                {"time_of_day", json::number(time_of_day)},
                {"seed", json::number(seed)},
                {"name", json::string(worldName)},
                {"creation_timestamp", json::string(timestamp())}
            });

            write_text_file(metadata_path, json::deparse(world_meta_template));
        } else {
            auto [val, errcode] = json::parse(read_text_file(metadata_path));
        }
    }

    void World::ChunkServerThread() {
        while (run_chunk_thread)
        {
            // TODO sleep the thread if requestedchunks is empty and wake it when pushing something into requestedchunks if it was empty.
            // Will cut down on needless busywaiting - Redacted.
            while (!RequestedChunks.empty())
            {
                Vector2 cell;
                RequestedChunks.front_pop(cell);

                if (HasChunkOnFile(cell)) {
                    ServedChunks.push(new Core::Chunk(cell, GetChunkFullPath(cell)));
                    continue;
                }
                auto* chunk = new Chunk(cell);
                generator.FirstPass(chunk);
                ServedChunks.push(chunk);
            }
        }
    }

    unsigned int World::GetCurrentEntityCount() const {
        return entities.size();
    }

    void World::RequestChunk(const Vector2 &cell) {
        if (!RequestedChunks.contains(cell) && std::find(chunks_in_waiting.begin(), chunks_in_waiting.end(), cell) == chunks_in_waiting.end())
        {
            RequestedChunks.push(cell);
            chunks_in_waiting.push_back(cell);
        }
    }

    bool World::AwaitingChunkAtCell(const Vector2& cell) const
    {
        int cnt = std::ranges::count(chunks_in_waiting, cell);

        if (cnt > 1)
        {
            // Not good.
            std::cerr << "Fuckiddy fuck" << std::endl;
            return false;
        }
        if (cnt == 1)
            return true;
        return false;
    }

    bool World::HasChunkOnFile(const Vector2 &cell) const {
        return std::filesystem::exists(worlds/world_name/"chunks"/cell.ToString());
    }

    std::filesystem::path World::GetWorldRootPath() const
    {
        return worlds/world_name;
    }

    std::filesystem::path World::GetChunkFullPath(const Vector2& cell) const
    {
        return GetWorldRootPath()/"chunks"/cell.ToString();
    }

    void World::AddEntity(Entity *e) {
        entities.push_back(std::move(e));
    }

    void World::SaveChunkToFile(const Vector2& cell, Chunk* chunk)
    {
        std::ofstream fout;
        fout.open(GetChunkFullPath(cell), std::ios::out | std::ios::binary);
        fout.write(reinterpret_cast<const char*>(chunk->ptr()), chunk->BufferSizeBytes());

        fout.close();
    }

    void World::RefreshAll() {

        namespace fs = std::filesystem;

        fs::path current_world_path = worlds/world_name;

        if (fs::exists(current_world_path))
            fs::remove_all(current_world_path);

        // clear doesn't actually delete the data pointed to.
        for (auto& c : loaded_chunks)
            delete c.second;

        loaded_chunks.clear();
    }


    void World::DoForcedTileTicks() {
        Tile* tile = nullptr;

        for (auto& [coords, chunk] : loaded_chunks) {

            chunk->SwapTileUpdateBuffers();

            for (int x = 0; x < Core::Chunk::ChunkSize; x++) {
                for (int y = 0; y < Core::Chunk::ChunkSize; y++) {
                    int wx = coords.x*Chunk::ChunkSize + x;
                    int wy = coords.y*Chunk::ChunkSize + y;

                    if (!chunk->GetTileUpdateBufferFlag(x, y))
                        continue;

                    chunk->SetTileUpdateBufferFlag(x, y, false);

                    TileID at = chunk->GetTile(x, y);

                    if (at == TileID::AIR)
                        continue;

                    tile = GetByNumeric(at);

                    if (tile != nullptr) {
                        if (tile->DoesForcedTicc()) {
                            tile->ForcedTicc(this, 0, wx, wy);
                            //chunk->SetTileUpdateFlag(x, y, true);
                        }
                    }
                }
            }

            //chunk->SwapTileUpdateBuffers();
        }
    }

    void World::DoRandomTileTicks() {
        // Every frame, In each loaded chunk, perform a RandomTileTick on N tiles.
        // Where N is RandomTileTickCoefficient.

        int max_tries = RandomTileTickCoefficient*5;
        int max_successes = RandomTileTickCoefficient;

        Tile* tile = nullptr;

        for (auto& [coords, chunk] : loaded_chunks)
        {
            for (int ticc = 0; ticc < RandomTileTickCoefficient; ticc++)
            {
                // Select random x,y coordinates between [0, ChunkSize]
                int x = rng.Int(0, Chunk::ChunkSize-1);
                int y = rng.Int(0, Chunk::ChunkSize-1);

                int wx = coords.x*Chunk::ChunkSize + x;
                int wy = coords.y*Chunk::ChunkSize + y;

                TileID at = chunk->GetTile(x, y);

                if (at == TileID::AIR)
                    continue;

                tile = GetByNumeric(at);

                /*
                if (tile != nullptr)
                    if (tile->DoesRandomTicc())
                        tile->RandomTicc(this, 0, wx, wy);
                        */
                if ((tile != nullptr) && (tile->DoesRandomTicc()))
                        tile->RandomTicc(this, 0, wx, wy);
            }
        }
    }

    void World::AutoSave() {
        Logs::Info("Autosaving...");
        for (auto& [coords, chunk] : loaded_chunks)
        {
            SaveChunkToFile(coords, chunk);
        }
        Logs::Info("Autosave successful!");
    }

    void World::SaveAndExit() {
        Logs::Info("Saving before exiting...");
        for (auto&[coords, chunk] : loaded_chunks)
        {
            SaveChunkToFile(coords, chunk);
        }
        Logs::Info("Saving successful!");
    }

    void World::DoRandomTileTick(const Vector2& coords, Chunk *chunk) {
        Tile* tile = nullptr;

        int max_tries = RandomTileTickCoefficient*5;
        int max_successes = RandomTileTickCoefficient;

        for (int ticc = 0; ticc < RandomTileTickCoefficient; ticc++)
        {
            // Select random x,y coordinates between [0, ChunkSize]
            int x = rng.Int(0, Chunk::ChunkSize-1);
            int y = rng.Int(0, Chunk::ChunkSize-1);

            int wx = coords.x*Chunk::ChunkSize + x;
            int wy = coords.y*Chunk::ChunkSize + y;

            TileID at = chunk->GetTile(x, y);

            if (at == TileID::AIR)
                continue;

            tile = GetByNumeric(at);

            /*
            if (tile != nullptr)
                if (tile->DoesRandomTicc())
                    tile->RandomTicc(this, 0, wx, wy);
                    */
            if ((tile != nullptr) && (tile->DoesRandomTicc()))
                tile->RandomTicc(this, 0, wx, wy);
        }
    }

    void World::DoForcedTileTick(const Vector2& coords, Chunk *chunk) {
        Tile* tile = nullptr;

        chunk->SwapTileUpdateBuffers();

        for (int x = 0; x < Core::Chunk::ChunkSize; x++) {
            for (int y = 0; y < Core::Chunk::ChunkSize; y++) {
                int wx = coords.x*Chunk::ChunkSize + x;
                int wy = coords.y*Chunk::ChunkSize + y;

                if (!chunk->GetTileUpdateBufferFlag(x, y))
                    continue;

                chunk->SetTileUpdateBufferFlag(x, y, false);

                TileID at = chunk->GetTile(x, y);

                if (at == TileID::AIR)
                    continue;

                tile = GetByNumeric(at);

                if (tile != nullptr) {
                    if (tile->DoesForcedTicc()) {
                        tile->ForcedTicc(this, 0, wx, wy);
                        //chunk->SetTileUpdateFlag(x, y, true);
                    }
                }
            }
        }
    }
}
